@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id$
  *
  * Project:  GDAL algorithms
  * Purpose:  Test alg
@@ -12,12 +11,15 @@
  ****************************************************************************/
 
 #include <array>
+#include <cmath>
+#include <limits>
 
 #include "gdal_unit_test.h"
 
 #include "cpl_conv.h"
 
 #include "gdal_alg.h"
+#include "gdal_alg_priv.h"
 #include "gdalwarper.h"
 #include "gdal_priv.h"
 
@@ -138,32 +140,34 @@ TEST_F(test_alg, GDALWarpResolveWorkingDataType_padfSrcNoDataReal)
 // GDALWarpResolveWorkingDataType: effect of padfSrcNoDataReal
 TEST_F(test_alg, GDALWarpResolveWorkingDataType_padfSrcNoDataReal_with_band)
 {
+    const auto lambda = [](GDALDataset *poDS)
+    {
+        std::unique_ptr<GDALWarpOptions, decltype(&GDALDestroyWarpOptions)>
+            psOptions(GDALCreateWarpOptions(), GDALDestroyWarpOptions);
+        psOptions->hSrcDS = GDALDataset::ToHandle(poDS);
+        psOptions->nBandCount = 1;
+        psOptions->panSrcBands =
+            static_cast<int *>(CPLMalloc(psOptions->nBandCount * sizeof(int)));
+        psOptions->panSrcBands[0] = 1;
+        psOptions->padfSrcNoDataReal =
+            static_cast<double *>(CPLMalloc(sizeof(double)));
+        psOptions->padfSrcNoDataReal[0] = 0.0;
+        GDALWarpResolveWorkingDataType(psOptions.get());
+        EXPECT_EQ(psOptions->eWorkingDataType, GDT_Byte);
+
+        psOptions->padfSrcNoDataReal[0] = -1.0;
+        GDALWarpResolveWorkingDataType(psOptions.get());
+        EXPECT_EQ(psOptions->eWorkingDataType, GDT_Byte);
+
+        psOptions->eWorkingDataType = GDT_Unknown;
+        psOptions->padfSrcNoDataReal[0] = 2.0;
+        GDALWarpResolveWorkingDataType(psOptions.get());
+        EXPECT_EQ(psOptions->eWorkingDataType, GDT_Byte);
+    };
+
     GDALDatasetUniquePtr poDS(GDALDriver::FromHandle(GDALGetDriverByName("MEM"))
                                   ->Create("", 1, 1, 1, GDT_Byte, nullptr));
-    GDALWarpOptions *psOptions = GDALCreateWarpOptions();
-    // False-positive: hSrcDS is no longer used after GDALDestroyWarpOptions()
-    // coverity[escape]
-    psOptions->hSrcDS = GDALDataset::ToHandle(poDS.get());
-    psOptions->nBandCount = 1;
-    psOptions->panSrcBands =
-        static_cast<int *>(CPLMalloc(psOptions->nBandCount * sizeof(int)));
-    psOptions->panSrcBands[0] = 1;
-    psOptions->padfSrcNoDataReal =
-        static_cast<double *>(CPLMalloc(sizeof(double)));
-    psOptions->padfSrcNoDataReal[0] = 0.0;
-    GDALWarpResolveWorkingDataType(psOptions);
-    EXPECT_EQ(psOptions->eWorkingDataType, GDT_Byte);
-
-    psOptions->padfSrcNoDataReal[0] = -1.0;
-    GDALWarpResolveWorkingDataType(psOptions);
-    EXPECT_EQ(psOptions->eWorkingDataType, GDT_Byte);
-
-    psOptions->eWorkingDataType = GDT_Unknown;
-    psOptions->padfSrcNoDataReal[0] = 2.0;
-    GDALWarpResolveWorkingDataType(psOptions);
-    EXPECT_EQ(psOptions->eWorkingDataType, GDT_Byte);
-
-    GDALDestroyWarpOptions(psOptions);
+    lambda(poDS.get());
 }
 
 // GDALWarpResolveWorkingDataType: effect of padfSrcNoDataImag
@@ -505,4 +509,46 @@ TEST_F(test_alg, GDALIsLineOfSightVisible_through_mountain)
                                          nullptr, nullptr, nullptr));
 }
 
+TEST_F(test_alg, GDALFloatAlmostEquals)
+{
+    float f = 1.23f;
+    EXPECT_TRUE(GDALFloatAlmostEquals(f, f));
+    EXPECT_TRUE(GDALFloatAlmostEquals(-f, -f));
+    EXPECT_FALSE(GDALFloatAlmostEquals(f, -f));
+    EXPECT_FALSE(GDALFloatAlmostEquals(f, 0.0f));
+    float f2 = std::nextafter(f, std::numeric_limits<float>::max());
+    EXPECT_TRUE(GDALFloatAlmostEquals(f, f2, 1));
+    EXPECT_TRUE(GDALFloatAlmostEquals(f2, f, 1));
+    EXPECT_TRUE(GDALFloatAlmostEquals(-f, -f2, 1));
+    EXPECT_TRUE(GDALFloatAlmostEquals(-f2, -f, 1));
+    float f3 = std::nextafter(f2, std::numeric_limits<float>::max());
+    EXPECT_FALSE(GDALFloatAlmostEquals(f, f3, 1));
+    EXPECT_FALSE(GDALFloatAlmostEquals(f3, f, 1));
+
+    EXPECT_TRUE(GDALFloatAlmostEquals(
+        std::nextafter(0.0f, std::numeric_limits<float>::max()),
+        std::nextafter(0.0f, -std::numeric_limits<float>::max())));
+}
+
+TEST_F(test_alg, GDALDoubleAlmostEquals)
+{
+    double f = 1.23;
+    EXPECT_TRUE(GDALDoubleAlmostEquals(f, f));
+    EXPECT_TRUE(GDALDoubleAlmostEquals(-f, -f));
+    EXPECT_FALSE(GDALDoubleAlmostEquals(f, -f));
+    EXPECT_FALSE(GDALDoubleAlmostEquals(f, 0));
+    double f2 = std::nextafter(f, std::numeric_limits<double>::max());
+    EXPECT_TRUE(GDALDoubleAlmostEquals(f, f2, 1));
+    EXPECT_TRUE(GDALDoubleAlmostEquals(f2, f, 1));
+    EXPECT_TRUE(GDALDoubleAlmostEquals(-f, -f2, 1));
+    EXPECT_TRUE(GDALDoubleAlmostEquals(-f2, -f, 1));
+    double f3 = std::nextafter(f2, std::numeric_limits<double>::max());
+    EXPECT_FALSE(GDALDoubleAlmostEquals(f, f3, 1));
+    EXPECT_FALSE(GDALDoubleAlmostEquals(f3, f, 1));
+
+    EXPECT_TRUE(GDALDoubleAlmostEquals(
+        std::nextafter(0, std::numeric_limits<double>::max()),
+        std::nextafter(0, -std::numeric_limits<double>::max())));
+}
+
 }  // namespace
