@@ -1,5 +1,4 @@
 /******************************************************************************
- * $Id$
  *
  * Project:  GDAL Core
  * Purpose:  Test GDALCopyWords().
@@ -12,11 +11,14 @@
  ****************************************************************************/
 
 #include "cpl_conv.h"
+#include "cpl_float.h"
 #include "gdal.h"
 
+#include <cmath>
 #include <cstdint>
 #include <iostream>
 #include <limits>
+#include <type_traits>
 
 #include "gtest_include.h"
 
@@ -25,16 +27,25 @@ namespace
 
 // ---------------------------------------------------------------------------
 
-template <class OutType, class ConstantType>
-void AssertRes(GDALDataType intype, ConstantType inval, GDALDataType outtype,
-               ConstantType expected_outval, OutType outval, int numLine)
+template <class OutType, class CT1, class CT2>
+void AssertRes(GDALDataType intype, CT1 inval, GDALDataType outtype,
+               CT2 expected_outval, OutType outval, int numLine)
 {
-    EXPECT_NEAR((double)outval, (double)expected_outval, 1.0)
-        << "Test failed at line " << numLine
-        << " (intype=" << GDALGetDataTypeName(intype)
-        << ",inval=" << (double)inval
-        << ",outtype=" << GDALGetDataTypeName(outtype) << ",got "
-        << (double)outval << " expected  " << expected_outval;
+    if (static_cast<double>(expected_outval) == static_cast<double>(outval) ||
+        (std::isnan(static_cast<double>(expected_outval)) &&
+         std::isnan(static_cast<double>(outval))))
+    {
+        // ok
+    }
+    else
+    {
+        EXPECT_NEAR((double)outval, (double)expected_outval, 1.0)
+            << "Test failed at line " << numLine
+            << " (intype=" << GDALGetDataTypeName(intype)
+            << ",inval=" << (double)inval
+            << ",outtype=" << GDALGetDataTypeName(outtype) << ",got "
+            << (double)outval << " expected  " << expected_outval;
+    }
 }
 
 #define MY_EXPECT(intype, inval, outtype, expected_outval, outval)             \
@@ -45,8 +56,8 @@ class TestCopyWords : public ::testing::Test
   protected:
     void SetUp() override
     {
-        pIn = (GByte *)malloc(256);
-        pOut = (GByte *)malloc(256);
+        pIn = (GByte *)malloc(2048);
+        pOut = (GByte *)malloc(2048);
     }
 
     void TearDown() override
@@ -64,8 +75,8 @@ class TestCopyWords : public ::testing::Test
               GDALDataType outtype, ConstantType outval, ConstantType outvali,
               int numLine)
     {
-        memset(pIn, 0xff, 128);
-        memset(pOut, 0xff, 128);
+        memset(pIn, 0xff, 1024);
+        memset(pOut, 0xff, 1024);
 
         *(InType *)(pIn) = (InType)inval;
         *(InType *)(pIn + 32) = (InType)inval;
@@ -79,14 +90,14 @@ class TestCopyWords : public ::testing::Test
         GDALCopyWords(pIn, intype, 32, pOut, outtype, 32, 2);
 
         /* Test negative offsets */
-        GDALCopyWords(pIn + 32, intype, -32, pOut + 128 - 16, outtype, -32, 2);
+        GDALCopyWords(pIn + 32, intype, -32, pOut + 1024 - 16, outtype, -32, 2);
 
         MY_EXPECT(intype, inval, outtype, outval, *(OutType *)(pOut));
         MY_EXPECT(intype, inval, outtype, outval, *(OutType *)(pOut + 32));
         MY_EXPECT(intype, inval, outtype, outval,
-                  *(OutType *)(pOut + 128 - 16));
+                  *(OutType *)(pOut + 1024 - 16));
         MY_EXPECT(intype, inval, outtype, outval,
-                  *(OutType *)(pOut + 128 - 16 - 32));
+                  *(OutType *)(pOut + 1024 - 16 - 32));
 
         if (GDALDataTypeIsComplex(outtype))
         {
@@ -95,38 +106,29 @@ class TestCopyWords : public ::testing::Test
                       ((OutType *)(pOut + 32))[1]);
 
             MY_EXPECT(intype, invali, outtype, outvali,
-                      ((OutType *)(pOut + 128 - 16))[1]);
+                      ((OutType *)(pOut + 1024 - 16))[1]);
             MY_EXPECT(intype, invali, outtype, outvali,
-                      ((OutType *)(pOut + 128 - 16 - 32))[1]);
+                      ((OutType *)(pOut + 1024 - 16 - 32))[1]);
         }
         else
         {
-            *(InType *)(pIn + GDALGetDataTypeSize(intype) / 8) = (InType)inval;
+            constexpr int N = 32 + 31;
+            for (int i = 0; i < N; ++i)
+            {
+                *(InType *)(pIn + i * GDALGetDataTypeSizeBytes(intype)) =
+                    (InType)inval;
+            }
+
             /* Test packed offsets */
-            GDALCopyWords(pIn, intype, GDALGetDataTypeSize(intype) / 8, pOut,
-                          outtype, GDALGetDataTypeSize(outtype) / 8, 2);
+            GDALCopyWords(pIn, intype, GDALGetDataTypeSizeBytes(intype), pOut,
+                          outtype, GDALGetDataTypeSizeBytes(outtype), N);
 
-            MY_EXPECT(intype, inval, outtype, outval, *(OutType *)(pOut));
-            MY_EXPECT(intype, inval, outtype, outval,
-                      *(OutType *)(pOut + GDALGetDataTypeSize(outtype) / 8));
-
-            *(InType *)(pIn + 2 * GDALGetDataTypeSize(intype) / 8) =
-                (InType)inval;
-            *(InType *)(pIn + 3 * GDALGetDataTypeSize(intype) / 8) =
-                (InType)inval;
-            /* Test packed offsets */
-            GDALCopyWords(pIn, intype, GDALGetDataTypeSize(intype) / 8, pOut,
-                          outtype, GDALGetDataTypeSize(outtype) / 8, 4);
-
-            MY_EXPECT(intype, inval, outtype, outval, *(OutType *)(pOut));
-            MY_EXPECT(intype, inval, outtype, outval,
-                      *(OutType *)(pOut + GDALGetDataTypeSize(outtype) / 8));
-            MY_EXPECT(
-                intype, inval, outtype, outval,
-                *(OutType *)(pOut + 2 * GDALGetDataTypeSize(outtype) / 8));
-            MY_EXPECT(
-                intype, inval, outtype, outval,
-                *(OutType *)(pOut + 3 * GDALGetDataTypeSize(outtype) / 8));
+            for (int i = 0; i < N; ++i)
+            {
+                MY_EXPECT(
+                    intype, inval, outtype, outval,
+                    *(OutType *)(pOut + i * GDALGetDataTypeSizeBytes(outtype)));
+            }
         }
     }
 
@@ -159,6 +161,9 @@ class TestCopyWords : public ::testing::Test
         else if (outtype == GDT_UInt64)
             Test<InType, std::uint64_t, ConstantType>(
                 intype, inval, invali, outtype, outval, outvali, numLine);
+        else if (outtype == GDT_Float16)
+            Test<InType, GFloat16, ConstantType>(intype, inval, invali, outtype,
+                                                 outval, outvali, numLine);
         else if (outtype == GDT_Float32)
             Test<InType, float, ConstantType>(intype, inval, invali, outtype,
                                               outval, outvali, numLine);
@@ -171,6 +176,9 @@ class TestCopyWords : public ::testing::Test
         else if (outtype == GDT_CInt32)
             Test<InType, GInt32, ConstantType>(intype, inval, invali, outtype,
                                                outval, outvali, numLine);
+        else if (outtype == GDT_CFloat16)
+            Test<InType, GFloat16, ConstantType>(intype, inval, invali, outtype,
+                                                 outval, outvali, numLine);
         else if (outtype == GDT_CFloat32)
             Test<InType, float, ConstantType>(intype, inval, invali, outtype,
                                               outval, outvali, numLine);
@@ -208,6 +216,9 @@ class TestCopyWords : public ::testing::Test
         else if (intype == GDT_UInt64)
             FromR_2<std::uint64_t, ConstantType>(intype, inval, invali, outtype,
                                                  outval, outvali, numLine);
+        else if (intype == GDT_Float16)
+            FromR_2<GFloat16, ConstantType>(intype, inval, invali, outtype,
+                                            outval, outvali, numLine);
         else if (intype == GDT_Float32)
             FromR_2<float, ConstantType>(intype, inval, invali, outtype, outval,
                                          outvali, numLine);
@@ -220,6 +231,9 @@ class TestCopyWords : public ::testing::Test
         else if (intype == GDT_CInt32)
             FromR_2<GInt32, ConstantType>(intype, inval, invali, outtype,
                                           outval, outvali, numLine);
+        else if (intype == GDT_CFloat16)
+            FromR_2<GFloat16, ConstantType>(intype, inval, invali, outtype,
+                                            outval, outvali, numLine);
         else if (intype == GDT_CFloat32)
             FromR_2<float, ConstantType>(intype, inval, invali, outtype, outval,
                                          outvali, numLine);
@@ -242,8 +256,8 @@ class TestCopyWords : public ::testing::Test
 #define IS_UNSIGNED(x)                                                         \
     (x == GDT_Byte || x == GDT_UInt16 || x == GDT_UInt32 || x == GDT_UInt64)
 #define IS_FLOAT(x)                                                            \
-    (x == GDT_Float32 || x == GDT_Float64 || x == GDT_CFloat32 ||              \
-     x == GDT_CFloat64)
+    (x == GDT_Float16 || x == GDT_Float32 || x == GDT_Float64 ||               \
+     x == GDT_CFloat16 || x == GDT_CFloat32 || x == GDT_CFloat64)
 
 #define CST_3000000000 (((GIntBig)3000) * 1000 * 1000)
 #define CST_5000000000 (((GIntBig)5000) * 1000 * 1000)
@@ -291,10 +305,12 @@ TEST_F(TestCopyWords, GDT_Int8)
     FROM_R(GDT_Int8, -128, GDT_UInt32, 0); /* clamp */
     FROM_R(GDT_Int8, -128, GDT_Int64, -128);
     FROM_R(GDT_Int8, -128, GDT_UInt64, 0); /* clamp */
+    FROM_R(GDT_Int8, -128, GDT_Float16, -128);
     FROM_R(GDT_Int8, -128, GDT_Float32, -128);
     FROM_R(GDT_Int8, -128, GDT_Float64, -128);
     FROM_R(GDT_Int8, -128, GDT_CInt16, -128);
     FROM_R(GDT_Int8, -128, GDT_CInt32, -128);
+    FROM_R(GDT_Int8, -128, GDT_CFloat16, -128);
     FROM_R(GDT_Int8, -128, GDT_CFloat32, -128);
     FROM_R(GDT_Int8, -128, GDT_CFloat64, -128);
     for (GDALDataType outtype = GDT_Byte; outtype < GDT_TypeCount;
@@ -508,6 +524,168 @@ TEST_F(TestCopyWords, GDT_UInt64)
     FROM_R(GDT_UInt64, nVal, GDT_CFloat64, nVal);
 }
 
+TEST_F(TestCopyWords, GDT_Float64)
+{
+    FROM_R_F(GDT_Float64, std::numeric_limits<double>::max(), GDT_Float32,
+             std::numeric_limits<double>::infinity());
+    FROM_R_F(GDT_Float64, -std::numeric_limits<double>::max(), GDT_Float32,
+             -std::numeric_limits<double>::infinity());
+    FROM_R_F(GDT_Float64, std::numeric_limits<double>::quiet_NaN(), GDT_Float32,
+             std::numeric_limits<double>::quiet_NaN());
+}
+
+TEST_F(TestCopyWords, GDT_Float16only)
+{
+    GDALDataType intype = GDT_Float16;
+    for (GDALDataType outtype = GDT_Byte; outtype < GDT_TypeCount;
+         outtype = (GDALDataType)(outtype + 1))
+    {
+        if (IS_FLOAT(outtype))
+        {
+            FROM_R_F(intype, 127.1, outtype, 127.1);
+            FROM_R_F(intype, -127.1, outtype, -127.1);
+        }
+        else
+        {
+            FROM_R_F(intype, 125.1, outtype, 125);
+            FROM_R_F(intype, 125.9, outtype, 126);
+
+            FROM_R_F(intype, 0.4, outtype, 0);
+            FROM_R_F(intype, 0.5, outtype,
+                     1); /* We could argue how to do this rounding */
+            FROM_R_F(intype, 0.6, outtype, 1);
+            FROM_R_F(intype, 126.5, outtype,
+                     127); /* We could argue how to do this rounding */
+
+            if (!IS_UNSIGNED(outtype))
+            {
+                FROM_R_F(intype, -125.9, outtype, -126);
+                FROM_R_F(intype, -127.1, outtype, -127);
+
+                FROM_R_F(intype, -0.4, outtype, 0);
+                FROM_R_F(intype, -0.5, outtype,
+                         -1); /* We could argue how to do this rounding */
+                FROM_R_F(intype, -0.6, outtype, -1);
+                FROM_R_F(intype, -127.5, outtype,
+                         -128); /* We could argue how to do this rounding */
+            }
+        }
+    }
+    FROM_R(intype, -30000, GDT_Byte, 0);
+    FROM_R(intype, -32768, GDT_Byte, 0);
+    FROM_R(intype, -1, GDT_Byte, 0);
+    FROM_R(intype, 256, GDT_Byte, 255);
+    FROM_R(intype, 30000, GDT_Byte, 255);
+    FROM_R(intype, -330000, GDT_Int16, -32768);
+    FROM_R(intype, -33000, GDT_Int16, -32768);
+    FROM_R(intype, 33000, GDT_Int16, 32767);
+    FROM_R(intype, -33000, GDT_UInt16, 0);
+    FROM_R(intype, -1, GDT_UInt16, 0);
+    FROM_R(intype, 60000, GDT_UInt16, 60000);
+    FROM_R(intype, -33000, GDT_Int32, -32992);
+    FROM_R(intype, 33000, GDT_Int32, 32992);
+    FROM_R(intype, -1, GDT_UInt32, 0);
+    FROM_R(intype, 60000, GDT_UInt32, 60000U);
+    FROM_R(intype, 33000, GDT_Float32, 32992);
+    FROM_R(intype, -33000, GDT_Float32, -32992);
+    FROM_R(intype, 33000, GDT_Float64, 32992);
+    FROM_R(intype, -33000, GDT_Float64, -32992);
+    FROM_R(intype, -33000, GDT_CInt16, -32768);
+    FROM_R(intype, 33000, GDT_CInt16, 32767);
+    FROM_R(intype, -33000, GDT_CInt32, -32992);
+    FROM_R(intype, 33000, GDT_CInt32, 32992);
+    FROM_R(intype, 33000, GDT_CFloat32, 32992);
+    FROM_R(intype, -33000, GDT_CFloat32, -32992);
+    FROM_R(intype, 33000, GDT_CFloat64, 32992);
+    FROM_R(intype, -33000, GDT_CFloat64, -32992);
+
+    FROM_R_F(GDT_Float32, std::numeric_limits<float>::max(), GDT_Float16,
+             std::numeric_limits<double>::infinity());
+    FROM_R_F(GDT_Float32, -std::numeric_limits<float>::max(), GDT_Float16,
+             -std::numeric_limits<double>::infinity());
+    FROM_R_F(GDT_Float32, std::numeric_limits<float>::quiet_NaN(), GDT_Float16,
+             std::numeric_limits<double>::quiet_NaN());
+
+    FROM_R_F(GDT_Float64, std::numeric_limits<double>::max(), GDT_Float16,
+             std::numeric_limits<double>::infinity());
+    FROM_R_F(GDT_Float64, -std::numeric_limits<double>::max(), GDT_Float16,
+             -std::numeric_limits<double>::infinity());
+    FROM_R_F(GDT_Float64, std::numeric_limits<double>::quiet_NaN(), GDT_Float16,
+             std::numeric_limits<double>::quiet_NaN());
+
+    // Float16 to Int64
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::quiet_NaN();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, 0);
+    }
+
+    {
+        GFloat16 in_value = -cpl::NumericLimits<GFloat16>::infinity();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, INT64_MIN);
+    }
+
+    {
+        GFloat16 in_value = -cpl::NumericLimits<GFloat16>::max();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, -65504);
+    }
+
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::max();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, 65504);
+    }
+
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::infinity();
+        int64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_Int64, 0, 1);
+        EXPECT_EQ(out_value, INT64_MAX);
+    }
+
+    // Float16 to UInt64
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::quiet_NaN();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, 0);
+    }
+
+    {
+        GFloat16 in_value = -cpl::NumericLimits<GFloat16>::infinity();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, 0);
+    }
+
+    {
+        GFloat16 in_value = -cpl::NumericLimits<GFloat16>::max();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, 0);
+    }
+
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::max();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, 65504);
+    }
+
+    {
+        GFloat16 in_value = cpl::NumericLimits<GFloat16>::infinity();
+        uint64_t out_value = 0;
+        GDALCopyWords(&in_value, GDT_Float16, 0, &out_value, GDT_UInt64, 0, 1);
+        EXPECT_EQ(out_value, UINT64_MAX);
+    }
+}
+
 TEST_F(TestCopyWords, GDT_Float32and64)
 {
     /* GDT_Float32 and GDT_Float64 */
@@ -582,35 +760,35 @@ TEST_F(TestCopyWords, GDT_Float32and64)
 
     // Float32 to Int64
     {
-        float in_value = std::numeric_limits<float>::quiet_NaN();
+        float in_value = cpl::NumericLimits<float>::quiet_NaN();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        float in_value = -std::numeric_limits<float>::infinity();
+        float in_value = -cpl::NumericLimits<float>::infinity();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MIN);
     }
 
     {
-        float in_value = -std::numeric_limits<float>::max();
+        float in_value = -cpl::NumericLimits<float>::max();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MIN);
     }
 
     {
-        float in_value = std::numeric_limits<float>::max();
+        float in_value = cpl::NumericLimits<float>::max();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MAX);
     }
 
     {
-        float in_value = std::numeric_limits<float>::infinity();
+        float in_value = cpl::NumericLimits<float>::infinity();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MAX);
@@ -618,35 +796,35 @@ TEST_F(TestCopyWords, GDT_Float32and64)
 
     // Float64 to Int64
     {
-        double in_value = std::numeric_limits<double>::quiet_NaN();
+        double in_value = cpl::NumericLimits<double>::quiet_NaN();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        double in_value = -std::numeric_limits<double>::infinity();
+        double in_value = -cpl::NumericLimits<double>::infinity();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MIN);
     }
 
     {
-        double in_value = -std::numeric_limits<double>::max();
+        double in_value = -cpl::NumericLimits<double>::max();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MIN);
     }
 
     {
-        double in_value = std::numeric_limits<double>::max();
+        double in_value = cpl::NumericLimits<double>::max();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MAX);
     }
 
     {
-        double in_value = std::numeric_limits<double>::infinity();
+        double in_value = cpl::NumericLimits<double>::infinity();
         int64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_Int64, 0, 1);
         EXPECT_EQ(out_value, INT64_MAX);
@@ -654,35 +832,35 @@ TEST_F(TestCopyWords, GDT_Float32and64)
 
     // Float32 to UInt64
     {
-        float in_value = std::numeric_limits<float>::quiet_NaN();
+        float in_value = cpl::NumericLimits<float>::quiet_NaN();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        float in_value = -std::numeric_limits<float>::infinity();
+        float in_value = -cpl::NumericLimits<float>::infinity();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        float in_value = -std::numeric_limits<float>::max();
+        float in_value = -cpl::NumericLimits<float>::max();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        float in_value = std::numeric_limits<float>::max();
+        float in_value = cpl::NumericLimits<float>::max();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, UINT64_MAX);
     }
 
     {
-        float in_value = std::numeric_limits<float>::infinity();
+        float in_value = cpl::NumericLimits<float>::infinity();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float32, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, UINT64_MAX);
@@ -690,35 +868,35 @@ TEST_F(TestCopyWords, GDT_Float32and64)
 
     // Float64 to UInt64
     {
-        double in_value = -std::numeric_limits<double>::quiet_NaN();
+        double in_value = -cpl::NumericLimits<double>::quiet_NaN();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        double in_value = -std::numeric_limits<double>::infinity();
+        double in_value = -cpl::NumericLimits<double>::infinity();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        double in_value = -std::numeric_limits<double>::max();
+        double in_value = -cpl::NumericLimits<double>::max();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, 0);
     }
 
     {
-        double in_value = std::numeric_limits<double>::max();
+        double in_value = cpl::NumericLimits<double>::max();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, UINT64_MAX);
     }
 
     {
-        double in_value = std::numeric_limits<double>::infinity();
+        double in_value = cpl::NumericLimits<double>::infinity();
         uint64_t out_value = 0;
         GDALCopyWords(&in_value, GDT_Float64, 0, &out_value, GDT_UInt64, 0, 1);
         EXPECT_EQ(out_value, UINT64_MAX);
@@ -843,6 +1021,49 @@ TEST_F(TestCopyWords, GDT_CFloat32and64)
     }
 }
 
+TEST_F(TestCopyWords, GDT_CFloat16only)
+{
+    GDALDataType intype = GDT_CFloat16;
+    for (GDALDataType outtype = GDT_Byte; outtype < GDT_TypeCount;
+         outtype = (GDALDataType)(outtype + 1))
+    {
+        if (IS_FLOAT(outtype))
+        {
+            FROM_C_F(intype, 127.1, 127.9, outtype, 127.1, 127.9);
+            FROM_C_F(intype, -127.1, -127.9, outtype, -127.1, -127.9);
+        }
+        else
+        {
+            FROM_C_F(intype, 126.1, 150.9, outtype, 126, 151);
+            FROM_C_F(intype, 126.9, 150.1, outtype, 127, 150);
+            if (!IS_UNSIGNED(outtype))
+            {
+                FROM_C_F(intype, -125.9, -127.1, outtype, -126, -127);
+            }
+        }
+    }
+    FROM_C(intype, -1, 256, GDT_Byte, 0, 0);
+    FROM_C(intype, 256, -1, GDT_Byte, 255, 0);
+    FROM_C(intype, -33000, 33000, GDT_Int16, -32768, 0);
+    FROM_C(intype, 33000, -33000, GDT_Int16, 32767, 0);
+    FROM_C(intype, -1, 66000, GDT_UInt16, 0, 0);
+    FROM_C(intype, 66000, -1, GDT_UInt16, 65535, 0);
+    FROM_C(intype, -33000, -33000, GDT_Int32, -32992, 0);
+    FROM_C(intype, 33000, 33000, GDT_Int32, 32992, 0);
+    FROM_C(intype, -1, 33000, GDT_UInt32, 0, 0);
+    FROM_C(intype, 33000, -1, GDT_UInt32, 32992, 0);
+    FROM_C(intype, 33000, -1, GDT_Float32, 32992, 0);
+    FROM_C(intype, 33000, -1, GDT_Float64, 32992, 0);
+    FROM_C(intype, -33000, -1, GDT_Float32, -32992, 0);
+    FROM_C(intype, -33000, -1, GDT_Float64, -32992, 0);
+    FROM_C(intype, -33000, 33000, GDT_CInt16, -32768, 32767);
+    FROM_C(intype, 33000, -33000, GDT_CInt16, 32767, -32768);
+    FROM_C(intype, -33000, -33000, GDT_CInt32, -32992, -32992);
+    FROM_C(intype, 33000, 33000, GDT_CInt32, 32992, 32992);
+    FROM_C(intype, 33000, -33000, GDT_CFloat32, 32992, -32992);
+    FROM_C(intype, 33000, -33000, GDT_CFloat64, 32992, -32992);
+}
+
 template <class Tin, class Tout>
 void CheckPackedGeneric(GDALDataType eIn, GDALDataType eOut)
 {
@@ -851,7 +1072,20 @@ void CheckPackedGeneric(GDALDataType eIn, GDALDataType eOut)
     Tout arrayOut[N];
     for (int i = 0; i < N; i++)
     {
-        arrayIn[i] = static_cast<Tin>(i + 1);
+        if constexpr (!std::is_integral_v<Tin> && std::is_integral_v<Tout>)
+        {
+            // Test correct rounding
+            if (i == 0 && std::is_unsigned_v<Tout>)
+                arrayIn[i] = cpl::NumericLimits<Tin>::quiet_NaN();
+            else if ((i % 2) != 0)
+                arrayIn[i] = static_cast<Tin>(i + 0.4);
+            else
+                arrayIn[i] = static_cast<Tin>(i + 0.6);
+        }
+        else
+        {
+            arrayIn[i] = static_cast<Tin>(i + 1);
+        }
         arrayOut[i] = 0;
     }
     GDALCopyWords(arrayIn, eIn, GDALGetDataTypeSizeBytes(eIn), arrayOut, eOut,
@@ -859,7 +1093,26 @@ void CheckPackedGeneric(GDALDataType eIn, GDALDataType eOut)
     int numLine = 0;
     for (int i = 0; i < N; i++)
     {
-        MY_EXPECT(eIn, i + 1, eOut, i + 1, arrayOut[i]);
+        if constexpr (!std::is_integral_v<Tin> && std::is_integral_v<Tout>)
+        {
+            if (i == 0 && std::is_unsigned_v<Tout>)
+            {
+                MY_EXPECT(eIn, cpl::NumericLimits<Tin>::quiet_NaN(), eOut, 0,
+                          arrayOut[i]);
+            }
+            else if ((i % 2) != 0)
+            {
+                MY_EXPECT(eIn, i + 0.4, eOut, i, arrayOut[i]);
+            }
+            else
+            {
+                MY_EXPECT(eIn, i + 0.6, eOut, i + 1, arrayOut[i]);
+            }
+        }
+        else
+        {
+            MY_EXPECT(eIn, i + 1, eOut, i + 1, arrayOut[i]);
+        }
     }
 }
 
@@ -946,6 +1199,9 @@ template <class Tin> void CheckPacked(GDALDataType eIn, GDALDataType eOut)
         case GDT_Int64:
             CheckPacked<Tin, std::int64_t>(eIn, eOut);
             break;
+        case GDT_Float16:
+            CheckPacked<Tin, GFloat16>(eIn, eOut);
+            break;
         case GDT_Float32:
             CheckPacked<Tin, float>(eIn, eOut);
             break;
@@ -985,6 +1241,9 @@ static void CheckPacked(GDALDataType eIn, GDALDataType eOut)
         case GDT_Int64:
             CheckPacked<std::int64_t>(eIn, eOut);
             break;
+        case GDT_Float16:
+            CheckPacked<GFloat16>(eIn, eOut);
+            break;
         case GDT_Float32:
             CheckPacked<float>(eIn, eOut);
             break;
