@@ -8,23 +8,7 @@
  * Copyright (c) 2017-2019, Even Rouault, <even dot rouault at spatialys dot
  *com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #ifndef GDALPYTHON_H_INCLUDED
@@ -124,59 +108,13 @@ struct PyMethodDef
     const char *help;
 };
 
-extern PyObject *(*PyModule_Create2)(struct PyModuleDef *, int);
+extern PyObject *(*PyCFunction_New)(const PyMethodDef *ml, PyObject *self);
+extern int (*PyModule_AddObject)(PyObject *mod, const char *name,
+                                 PyObject *value);
 
-#define PYTHON_API_VERSION 1013
-
-/* Flag passed to newmethodobject */
 #define METH_VARARGS 0x0001
 #define METH_KEYWORDS 0x0002
 
-#define _PyObject_HEAD_EXTRA
-
-struct _object
-{
-    _PyObject_HEAD_EXTRA Py_ssize_t ob_refcnt;
-    void /*struct _typeobject*/ *ob_type;
-};
-
-#define PyObject_HEAD PyObject ob_base;
-
-#define _PyObject_EXTRA_INIT
-
-#define PyObject_HEAD_INIT(type) {_PyObject_EXTRA_INIT 1, type},
-
-#define PyModuleDef_HEAD_INIT                                                  \
-    {                                                                          \
-        PyObject_HEAD_INIT(nullptr) nullptr, /* m_init */                      \
-            0,                               /* m_index */                     \
-            nullptr,                         /* m_copy */                      \
-    }
-
-typedef struct PyModuleDef_Base
-{
-    PyObject_HEAD PyObject *(*m_init)(void);
-    Py_ssize_t m_index;
-    PyObject *m_copy;
-} PyModuleDef_Base;
-
-typedef void *traverseproc;
-typedef void *inquiry;
-typedef void *freefunc;
-
-typedef struct PyModuleDef
-{
-    PyModuleDef_Base m_base;
-    const char *m_name;
-    const char *m_doc;
-    Py_ssize_t m_size;
-    const PyMethodDef *m_methods;
-    struct PyModuleDef_Slot *m_slots;
-    traverseproc m_traverse;
-    inquiry m_clear;
-    freefunc m_free;
-} PyModuleDef;
-
 #define Py_file_input 257
 
 typedef int PyGILState_STATE;
