@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test /vsizip/vsimem/
@@ -13,7 +12,9 @@
 # SPDX-License-Identifier: MIT
 ###############################################################################
 
+import os
 import random
+import sys
 
 import gdaltest
 import pytest
@@ -208,31 +209,38 @@ def test_vsizip_2():
 # Test opening in write mode a file inside a zip archive whose content has been listed before (testcase for fix of r22625)
 
 
-def test_vsizip_3():
+def test_vsizip_3(tmp_path):
 
-    fmain = gdal.VSIFOpenL("/vsizip/vsimem/test3.zip", "wb")
+    fds_open = 0
+    if sys.platform == "linux":
+        fds_open = len(os.listdir("/proc/self/fd"))
 
-    f = gdal.VSIFOpenL("/vsizip/vsimem/test3.zip/foo", "wb")
+    filename_base = f"/vsizip/{tmp_path}/test3.zip"
+
+    fmain = gdal.VSIFOpenL(filename_base, "wb")
+
+    f = gdal.VSIFOpenL(filename_base + "/foo", "wb")
     gdal.VSIFWriteL("foo", 1, 3, f)
     gdal.VSIFCloseL(f)
-    f = gdal.VSIFOpenL("/vsizip/vsimem/test3.zip/bar", "wb")
+    f = gdal.VSIFOpenL(filename_base + "/bar", "wb")
     gdal.VSIFWriteL("bar", 1, 3, f)
     gdal.VSIFCloseL(f)
 
     gdal.VSIFCloseL(fmain)
 
-    gdal.ReadDir("/vsizip/vsimem/test3.zip")
+    gdal.ReadDir(filename_base)
 
-    f = gdal.VSIFOpenL("/vsizip/vsimem/test3.zip/baz", "wb")
+    f = gdal.VSIFOpenL(filename_base + "/baz", "wb")
     gdal.VSIFWriteL("baz", 1, 3, f)
     gdal.VSIFCloseL(f)
 
-    res = gdal.ReadDir("/vsizip/vsimem/test3.zip")
-
-    gdal.Unlink("/vsimem/test3.zip")
+    res = gdal.ReadDir(filename_base)
 
     assert res == ["foo", "bar", "baz"]
 
+    if sys.platform == "linux":
+        assert len(os.listdir("/proc/self/fd")) == fds_open
+
 
 ###############################################################################
 # Test ReadRecursive on valid zip
@@ -911,11 +919,10 @@ def test_vsizip_byte_copyfile_file_already_open():
 ###############################################################################
 
 
-def test_vsizip_byte_sozip():
-
-    zipfilename = "/vsimem/test_vsizip_byte_sozip.zip"
-    dstfilename = f"/vsizip/{zipfilename}/test.tif"
-    try:
+def test_vsizip_byte_sozip(tmp_path):
+    def do():
+        zipfilename = f"{tmp_path}/test_vsizip_byte_sozip.zip"
+        dstfilename = f"/vsizip/{zipfilename}/test.tif"
         options = ["SOZIP_ENABLED=YES", "SOZIP_CHUNK_SIZE=128"]
         assert gdal.CopyFile("data/byte.tif", dstfilename, options=options) == 0
         assert gdal.VSIStatL(dstfilename).size == gdal.VSIStatL("data/byte.tif").size
@@ -928,9 +935,16 @@ def test_vsizip_byte_sozip():
 
         ds = gdal.Open(dstfilename)
         assert ds.GetRasterBand(1).Checksum() == 4672
+        ds.Close()
 
-    finally:
-        gdal.Unlink(zipfilename)
+    fds_open = 0
+    if sys.platform == "linux":
+        fds_open = len(os.listdir("/proc/self/fd"))
+
+    do()
+
+    if sys.platform == "linux":
+        assert len(os.listdir("/proc/self/fd")) == fds_open
 
 
 ###############################################################################
@@ -961,3 +975,37 @@ def test_vsizip_sozip_of_file_bigger_than_4GB():
         assert gdal.VSIFReadL(1, 2, f) == b"\x00"
     finally:
         gdal.VSIFCloseL(f)
+
+
+###############################################################################
+# Test bugfix for https://github.com/OSGeo/gdal/issues/12572
+
+
+@pytest.mark.require_curl()
+def test_vsizip_vsicurl_error():
+
+    gdal.VSIErrorReset()
+    gdal.VSIStatL(
+        "/vsicurl/https://expired-rsa-dv.ssl.com/",
+        gdal.VSI_STAT_EXISTS_FLAG
+        | gdal.VSI_STAT_NATURE_FLAG
+        | gdal.VSI_STAT_SIZE_FLAG
+        | gdal.VSI_STAT_SET_ERROR_FLAG,
+    )
+    if "server certificate verification failed" not in gdal.VSIGetLastErrorMsg():
+        pytest.skip(
+            "Expected 'server certificate verification failed' in "
+            + gdal.VSIGetLastErrorMsg()
+        )
+
+    gdal.VSICurlClearCache()
+
+    gdal.VSIErrorReset()
+    gdal.VSIStatL(
+        "/vsizip/{/vsicurl/https://expired-rsa-dv.ssl.com/}",
+        gdal.VSI_STAT_EXISTS_FLAG
+        | gdal.VSI_STAT_NATURE_FLAG
+        | gdal.VSI_STAT_SIZE_FLAG
+        | gdal.VSI_STAT_SET_ERROR_FLAG,
+    )
+    assert "server certificate verification failed" in gdal.VSIGetLastErrorMsg()
