@@ -1,8 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id $
-#
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test KEA driver
 # Author:   Even Rouault, <even dot rouault at spatialys dot com>
@@ -10,29 +8,15 @@
 ###############################################################################
 # Copyright (c) 2014, Even Rouault <even dot rouault at spatialys dot com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
+import datetime
+
 import gdaltest
 import pytest
 
-from osgeo import gdal, osr
+from osgeo import gdal, ogr, osr
 
 pytestmark = pytest.mark.require_driver("KEA")
 
@@ -474,7 +458,7 @@ def test_kea_11(tmp_path):
 # Test RAT
 
 
-def test_kea_12(tmp_path):
+def test_kea_rat(tmp_path):
 
     ds = gdaltest.kea_driver.Create(tmp_path / "out.kea", 1, 1, 1, gdal.GDT_Byte)
     assert ds.GetRasterBand(1).GetDefaultRAT().GetColumnCount() == 0
@@ -489,10 +473,16 @@ def test_kea_12(tmp_path):
     rat.CreateColumn("col_integer_green", gdal.GFT_Integer, gdal.GFU_Green)
     rat.CreateColumn("col_integer_blue", gdal.GFT_Integer, gdal.GFU_Blue)
     rat.CreateColumn("col_integer_alpha", gdal.GFT_Integer, gdal.GFU_Alpha)
+    rat.CreateColumn("col_bool", gdal.GFT_Boolean, gdal.GFU_Generic)
+    rat.CreateColumn("col_datetime", gdal.GFT_DateTime, gdal.GFU_Generic)
+    rat.CreateColumn("col_wkbgeneric", gdal.GFT_WKBGeometry, gdal.GFU_Generic)
     rat.SetRowCount(1)
 
     rat.SetValueAsString(0, 0, "1.23")
     rat.SetValueAsInt(0, 0, 1)
+    rat.SetValueAsBoolean(0, 0, False)
+    rat.SetValueAsDateTime(0, 0, datetime.datetime.fromtimestamp(0))
+    rat.SetValueAsWKBGeometry(0, 0, b"")
     rat.SetValueAsDouble(0, 0, 1.23)
 
     rat.SetValueAsInt(0, 2, 0)
@@ -503,6 +493,41 @@ def test_kea_12(tmp_path):
     rat.SetValueAsDouble(0, 3, 123)
     rat.SetValueAsInt(0, 3, 123)
 
+    rat.SetValueAsString(0, 7, "true")
+    rat.SetValueAsDouble(0, 7, 0)
+    rat.SetValueAsInt(0, 7, 1)
+    rat.SetValueAsDateTime(0, 7, datetime.datetime.fromtimestamp(0))
+    rat.SetValueAsWKBGeometry(0, 7, b"")
+    rat.SetValueAsBoolean(0, 7, True)
+
+    rat.SetValueAsString(0, 8, "true")
+    rat.SetValueAsDouble(0, 8, 0)
+    rat.SetValueAsInt(0, 8, 1)
+    rat.SetValueAsWKBGeometry(0, 8, b"")
+    rat.SetValueAsBoolean(0, 8, True)
+    dt = datetime.datetime(
+        2025,
+        10,
+        11,
+        12,
+        34,
+        56,
+        500000,
+        tzinfo=datetime.timezone(datetime.timedelta(seconds=4500)),
+    )
+    rat.SetValueAsDateTime(0, 8, dt)
+
+    g = ogr.Geometry(ogr.wkbPoint)
+    g.SetPoint_2D(0, 1, 2)
+    wkb = g.ExportToWkb()
+
+    rat.SetValueAsString(0, 9, "true")
+    rat.SetValueAsDouble(0, 9, 0)
+    rat.SetValueAsInt(0, 9, 1)
+    rat.SetValueAsDateTime(0, 9, datetime.datetime.fromtimestamp(0))
+    rat.SetValueAsBoolean(0, 9, True)
+    rat.SetValueAsWKBGeometry(0, 9, wkb)
+
     cloned_rat = rat.Clone()
     assert ds.GetRasterBand(1).SetDefaultRAT(rat) == 0
     ds = None
@@ -525,7 +550,10 @@ def test_kea_12(tmp_path):
     assert rat.GetRowCount() == cloned_rat.GetRowCount()
     for i in range(rat.GetColumnCount()):
         assert rat.GetNameOfCol(i) == cloned_rat.GetNameOfCol(i)
-        assert rat.GetTypeOfCol(i) == cloned_rat.GetTypeOfCol(i)
+        if rat.GetTypeOfCol(i) not in (gdal.GFT_DateTime, gdal.GFT_WKBGeometry):
+            assert rat.GetTypeOfCol(i) == cloned_rat.GetTypeOfCol(i), rat.GetNameOfCol(
+                i
+            )
         assert rat.GetUsageOfCol(i) == cloned_rat.GetUsageOfCol(i)
 
     with gdal.quiet_errors():
@@ -544,6 +572,12 @@ def test_kea_12(tmp_path):
             rat.GetValueAsInt(-1, 0)
         with pytest.raises(Exception):
             rat.GetValueAsString(-1, 0)
+        with pytest.raises(Exception):
+            rat.GetValueAsBoolean(-1, 0)
+        with pytest.raises(Exception):
+            rat.GetValueAsDateTime(-1, 0)
+        with pytest.raises(Exception):
+            rat.GetValueAsWKBGeometry(-1, 0)
 
         with pytest.raises(Exception):
             rat.GetValueAsDouble(rat.GetColumnCount(), 0)
@@ -551,6 +585,12 @@ def test_kea_12(tmp_path):
             rat.GetValueAsInt(rat.GetColumnCount(), 0)
         with pytest.raises(Exception):
             rat.GetValueAsString(rat.GetColumnCount(), 0)
+        with pytest.raises(Exception):
+            rat.GetValueAsBoolean(rat.GetColumnCount(), 0)
+        with pytest.raises(Exception):
+            rat.GetValueAsDateTime(rat.GetColumnCount(), 0)
+        with pytest.raises(Exception):
+            rat.GetValueAsWKBGeometry(rat.GetColumnCount(), 0)
 
         with pytest.raises(Exception):
             rat.GetValueAsDouble(0, -1)
@@ -558,10 +598,19 @@ def test_kea_12(tmp_path):
             rat.GetValueAsInt(0, -1)
         with pytest.raises(Exception):
             rat.GetValueAsString(0, -1)
+        with pytest.raises(Exception):
+            rat.GetValueAsBoolean(0, -1)
+        with pytest.raises(Exception):
+            rat.GetValueAsDateTime(0, -1)
+        with pytest.raises(Exception):
+            rat.GetValueAsWKBGeometry(0, -1)
 
         rat.GetValueAsDouble(0, rat.GetRowCount())
         rat.GetValueAsInt(0, rat.GetRowCount())
         rat.GetValueAsString(0, rat.GetRowCount())
+        rat.GetValueAsBoolean(0, rat.GetRowCount())
+        rat.GetValueAsDateTime(0, rat.GetRowCount())
+        rat.GetValueAsWKBGeometry(0, rat.GetRowCount())
 
     assert rat.GetValueAsDouble(0, 0) == 1.23
     assert rat.GetValueAsInt(0, 0) == 1
@@ -575,6 +624,10 @@ def test_kea_12(tmp_path):
     assert rat.GetValueAsInt(0, 2) == 0
     assert rat.GetValueAsDouble(0, 2) == 0
 
+    assert rat.GetValueAsBoolean(0, 7)
+    assert rat.GetValueAsDateTime(0, 8) == dt
+    assert rat.GetValueAsWKBGeometry(0, 9) == wkb
+
     ds = None
     out2_ds = None
 
