@@ -93,7 +93,8 @@ CPLString OGRCARTOEscapeLiteral(const char *pszStr)
 char *OGRCARTOTableLayer::OGRCARTOGetHexGeometry(OGRGeometry *poGeom, int i)
 {
     OGRCartoGeomFieldDefn *poGeomFieldDefn =
-        (OGRCartoGeomFieldDefn *)(poFeatureDefn->GetGeomFieldDefn(i));
+        cpl::down_cast<OGRCartoGeomFieldDefn *>(
+            poFeatureDefn->GetGeomFieldDefn(i));
     int nSRID = poGeomFieldDefn->nSRID;
     if (nSRID == 0)
         nSRID = 4326;
@@ -432,22 +433,13 @@ OGRErr OGRCARTOTableLayer::SetAttributeFilter(const char *pszQuery)
 }
 
 /************************************************************************/
-/*                          SetSpatialFilter()                          */
+/*                          ISetSpatialFilter()                         */
 /************************************************************************/
 
-void OGRCARTOTableLayer::SetSpatialFilter(int iGeomField, OGRGeometry *poGeomIn)
+OGRErr OGRCARTOTableLayer::ISetSpatialFilter(int iGeomField,
+                                             const OGRGeometry *poGeomIn)
 
 {
-    if (iGeomField < 0 || iGeomField >= GetLayerDefn()->GetGeomFieldCount() ||
-        GetLayerDefn()->GetGeomFieldDefn(iGeomField)->GetType() == wkbNone)
-    {
-        if (iGeomField != 0)
-        {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                     "Invalid geometry field index : %d", iGeomField);
-        }
-        return;
-    }
     m_iGeomFieldFilter = iGeomField;
 
     if (InstallFilter(poGeomIn))
@@ -456,6 +448,8 @@ void OGRCARTOTableLayer::SetSpatialFilter(int iGeomField, OGRGeometry *poGeomIn)
 
         ResetReading();
     }
+
+    return OGRERR_NONE;
 }
 
 /************************************************************************/
@@ -1478,7 +1472,8 @@ OGRErr OGRCARTOTableLayer::ISetFeature(OGRFeature *poFeature)
         else
         {
             OGRCartoGeomFieldDefn *poGeomFieldDefn =
-                (OGRCartoGeomFieldDefn *)(poFeatureDefn->GetGeomFieldDefn(i));
+                cpl::down_cast<OGRCartoGeomFieldDefn *>(
+                    poFeatureDefn->GetGeomFieldDefn(i));
             int nSRID = poGeomFieldDefn->nSRID;
             if (nSRID == 0)
                 nSRID = 4326;
@@ -1733,14 +1728,14 @@ GIntBig OGRCARTOTableLayer::GetFeatureCount(int bForce)
 }
 
 /************************************************************************/
-/*                             GetExtent()                              */
+/*                            IGetExtent()                              */
 /*                                                                      */
 /*      For PostGIS use internal Extend(geometry) function              */
-/*      in other cases we use standard OGRLayer::GetExtent()            */
+/*      in other cases we use standard OGRLayer::IGetExtent()           */
 /************************************************************************/
 
-OGRErr OGRCARTOTableLayer::GetExtent(int iGeomField, OGREnvelope *psExtent,
-                                     int bForce)
+OGRErr OGRCARTOTableLayer::IGetExtent(int iGeomField, OGREnvelope *psExtent,
+                                      bool bForce)
 {
     CPLString osSQL;
 
@@ -1749,17 +1744,6 @@ OGRErr OGRCARTOTableLayer::GetExtent(int iGeomField, OGREnvelope *psExtent,
     if (FlushDeferredBuffer() != OGRERR_NONE)
         return OGRERR_FAILURE;
 
-    if (iGeomField < 0 || iGeomField >= GetLayerDefn()->GetGeomFieldCount() ||
-        GetLayerDefn()->GetGeomFieldDefn(iGeomField)->GetType() == wkbNone)
-    {
-        if (iGeomField != 0)
-        {
-            CPLError(CE_Failure, CPLE_AppDefined,
-                     "Invalid geometry field index : %d", iGeomField);
-        }
-        return OGRERR_FAILURE;
-    }
-
     OGRGeomFieldDefn *poGeomFieldDefn =
         poFeatureDefn->GetGeomFieldDefn(iGeomField);
 
@@ -1835,17 +1819,14 @@ OGRErr OGRCARTOTableLayer::GetExtent(int iGeomField, OGREnvelope *psExtent,
     if (poObj != nullptr)
         json_object_put(poObj);
 
-    if (iGeomField == 0)
-        return OGRLayer::GetExtent(psExtent, bForce);
-    else
-        return OGRLayer::GetExtent(iGeomField, psExtent, bForce);
+    return OGRLayer::IGetExtent(iGeomField, psExtent, bForce);
 }
 
 /************************************************************************/
 /*                           TestCapability()                           */
 /************************************************************************/
 
-int OGRCARTOTableLayer::TestCapability(const char *pszCap)
+int OGRCARTOTableLayer::TestCapability(const char *pszCap) const
 
 {
     if (EQUAL(pszCap, OLCFastFeatureCount))
@@ -1929,7 +1910,8 @@ OGRErr OGRCARTOTableLayer::RunDeferredCreationIfNecessary()
     for (int i = 0; i < poFeatureDefn->GetGeomFieldCount(); i++)
     {
         OGRCartoGeomFieldDefn *poFieldDefn =
-            (OGRCartoGeomFieldDefn *)(poFeatureDefn->GetGeomFieldDefn(i));
+            cpl::down_cast<OGRCartoGeomFieldDefn *>(
+                poFeatureDefn->GetGeomFieldDefn(i));
         OGRwkbGeometryType eGType = poFieldDefn->GetType();
         if (eGType == wkbNone)
             continue;
