@@ -11,23 +11,7 @@
  * Copyright (c) 1999-2001, Daniel Morissette
  * Copyright (c) 2014, Even Rouault <even.rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  **********************************************************************/
 
 #include "cpl_port.h"
@@ -37,9 +21,6 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
-#if HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
 #include <algorithm>
 #include <string>
 
@@ -223,6 +204,12 @@ int TABDATFile::Open(const char *pszFname, TABAccess eAccess,
             m_poHeaderBlock->GotoByteInFile((i + 1) * 32);
             m_poHeaderBlock->ReadBytes(
                 11, reinterpret_cast<GByte *>(m_pasFieldDef[i].szName));
+            constexpr char HEADER_RECORD_TERMINATOR = 0x0D;
+            if (m_pasFieldDef[i].szName[0] == HEADER_RECORD_TERMINATOR)
+            {
+                m_numFields = i;
+                break;
+            }
             m_pasFieldDef[i].szName[10] = '\0';
             m_pasFieldDef[i].cType =
                 static_cast<char>(m_poHeaderBlock->ReadByte());
@@ -748,19 +735,20 @@ static int TABDATFileSetFieldDefinition(TABDATFieldDef *psFieldDef,
     else if (nWidth == 0)
         nWidth = 254;  // char fields.
 
-    strncpy(psFieldDef->szName, pszName, sizeof(psFieldDef->szName) - 1);
-    psFieldDef->szName[sizeof(psFieldDef->szName) - 1] = '\0';
+    snprintf(psFieldDef->szName, sizeof(psFieldDef->szName), "%s", pszName);
     psFieldDef->eTABType = eType;
-    psFieldDef->byLength = static_cast<GByte>(nWidth);
-    psFieldDef->byDecimals = static_cast<GByte>(nPrecision);
+    psFieldDef->byDecimals = 0;
 
     switch (eType)
     {
         case TABFChar:
             psFieldDef->cType = 'C';
+            psFieldDef->byLength = static_cast<GByte>(nWidth);
             break;
         case TABFDecimal:
             psFieldDef->cType = 'N';
+            psFieldDef->byLength = static_cast<GByte>(nWidth);
+            psFieldDef->byDecimals = static_cast<GByte>(nPrecision);
             break;
         case TABFInteger:
             psFieldDef->cType = 'C';
@@ -1236,8 +1224,8 @@ int TABDATFile::ReorderFields(int *panMap)
 /*                           AlterFieldDefn()                           */
 /************************************************************************/
 
-int TABDATFile::AlterFieldDefn(int iField, OGRFieldDefn *poNewFieldDefn,
-                               int nFlags)
+int TABDATFile::AlterFieldDefn(int iField, const OGRFieldDefn *poSrcFieldDefn,
+                               OGRFieldDefn *poNewFieldDefn, int nFlags)
 {
     if (m_fp == nullptr)
     {
@@ -1261,20 +1249,20 @@ int TABDATFile::AlterFieldDefn(int iField, OGRFieldDefn *poNewFieldDefn,
     }
 
     TABFieldType eTABType = m_pasFieldDef[iField].eTABType;
-    int nWidth = m_pasFieldDef[iField].byLength;
-    int nPrecision = m_pasFieldDef[iField].byDecimals;
-    int nWidthDummy = 0;
-    int nPrecisionDummy = 0;
+    int nWidth = poSrcFieldDefn->GetWidth();
+    int nPrecision = poSrcFieldDefn->GetPrecision();
     if (nFlags & ALTER_TYPE_FLAG)
     {
-        if (IMapInfoFile::GetTABType(poNewFieldDefn, &eTABType, &nWidthDummy,
-                                     &nPrecisionDummy) < 0)
+        if (IMapInfoFile::GetTABType(poNewFieldDefn, &eTABType, nullptr,
+                                     nullptr) < 0)
             return -1;
     }
     if (nFlags & ALTER_WIDTH_PRECISION_FLAG)
     {
-        TABFieldType eTABTypeDummy;
-        if (IMapInfoFile::GetTABType(poNewFieldDefn, &eTABTypeDummy, &nWidth,
+        // Instead of taking directly poNewFieldDefn->GetWidth()/GetPrecision(),
+        // use GetTABType() to take into account .dat limitations on
+        // width & precision to clamp what user might have specify
+        if (IMapInfoFile::GetTABType(poNewFieldDefn, nullptr, &nWidth,
                                      &nPrecision) < 0)
             return -1;
     }
@@ -1282,22 +1270,24 @@ int TABDATFile::AlterFieldDefn(int iField, OGRFieldDefn *poNewFieldDefn,
     if ((nFlags & ALTER_TYPE_FLAG) &&
         eTABType != m_pasFieldDef[iField].eTABType)
     {
-        if (eTABType != TABFChar)
+        if (eTABType != TABFChar && m_numRecords > 0)
         {
             CPLError(CE_Failure, CPLE_NotSupported,
                      "Can only convert to OFTString");
             return -1;
         }
-        if ((nFlags & ALTER_WIDTH_PRECISION_FLAG) == 0)
+        if (eTABType == TABFChar && (nFlags & ALTER_WIDTH_PRECISION_FLAG) == 0)
             nWidth = 254;
     }
 
     if (nFlags & ALTER_WIDTH_PRECISION_FLAG)
     {
-        if (eTABType != TABFChar && nWidth != m_pasFieldDef[iField].byLength)
+        if (eTABType != TABFChar && nWidth != poSrcFieldDefn->GetWidth() &&
+            m_numRecords > 0)
         {
-            CPLError(CE_Failure, CPLE_NotSupported,
-                     "Resizing only supported on String fields");
+            CPLError(
+                CE_Failure, CPLE_NotSupported,
+                "Resizing only supported on String fields on non-empty layer");
             return -1;
         }
     }
@@ -1330,12 +1320,31 @@ int TABDATFile::AlterFieldDefn(int iField, OGRFieldDefn *poNewFieldDefn,
         }
         if (nFlags & ALTER_WIDTH_PRECISION_FLAG)
         {
-            m_pasFieldDef[iField].byLength = static_cast<GByte>(nWidth);
-            m_pasFieldDef[iField].byDecimals = static_cast<GByte>(nPrecision);
+            if (eTABType == TABFChar || eTABType == TABFDecimal)
+                m_pasFieldDef[iField].byLength = static_cast<GByte>(nWidth);
+            if (eTABType == TABFDecimal)
+                m_pasFieldDef[iField].byDecimals =
+                    static_cast<GByte>(nPrecision);
         }
         return 0;
     }
 
+    const bool bWidthPrecisionPreserved =
+        (nWidth == poSrcFieldDefn->GetWidth() &&
+         nPrecision == poSrcFieldDefn->GetPrecision());
+    if (eTABType == m_pasFieldDef[iField].eTABType && bWidthPrecisionPreserved)
+    {
+        return 0;
+    }
+
+    if (eTABType != TABFChar)
+    {
+        // should hopefully not happen given all above checks
+        CPLError(CE_Failure, CPLE_NotSupported,
+                 "Unsupported AlterFieldDefn() operation");
+        return -1;
+    }
+
     // Otherwise we need to do a temporary file.
     TABDATFile oTempFile(GetEncoding());
     CPLString osOriginalFile(m_pszFname);
@@ -1442,7 +1451,8 @@ int TABDATFile::AlterFieldDefn(int iField, OGRFieldDefn *poNewFieldDefn,
             else if (m_pasFieldDef[iField].eTABType == TABFLogical)
             {
                 strncpy(pabyNewField,
-                        ReadLogicalField(m_pasFieldDef[iField].byLength),
+                        ReadLogicalField(m_pasFieldDef[iField].byLength) ? "T"
+                                                                         : "F",
                         sFieldDef.byLength);
             }
             else if (m_pasFieldDef[iField].eTABType == TABFDate)
@@ -1745,25 +1755,22 @@ double TABDATFile::ReadFloatField(int nWidth)
  * Read the logical field value at the current position in the data
  * block.
  *
- * The file contains either 0 or 1, and we return a string with
- * "F" (false) or "T" (true)
- *
  * Note: nWidth is used only with TABTableDBF types.
  *
  * CPLError() will have been called if something fails.
  **********************************************************************/
-const char *TABDATFile::ReadLogicalField(int nWidth)
+bool TABDATFile::ReadLogicalField(int nWidth)
 {
     // If current record has been deleted, then return an acceptable
     // default value.
     if (m_bCurRecordDeletedFlag)
-        return "F";
+        return false;
 
     if (m_poRecordBlock == nullptr)
     {
         CPLError(CE_Failure, CPLE_AssertionFailed,
                  "Can't read field value: file is not opened.");
-        return "";
+        return false;
     }
 
     bool bValue = false;
@@ -1778,7 +1785,7 @@ const char *TABDATFile::ReadLogicalField(int nWidth)
         bValue = CPL_TO_BOOL(m_poRecordBlock->ReadByte());
     }
 
-    return bValue ? "T" : "F";
+    return bValue;
 }
 
 /**********************************************************************
@@ -2217,12 +2224,11 @@ int TABDATFile::WriteFloatField(double dValue, TABINDFile *poINDFile,
  * Write the logical field value at the current position in the data
  * block.
  *
- * The value written to the file is either 0 or 1, but this function
- * takes as input a string with "F" (false) or "T" (true)
+ * The value written to the file is either 0 or 1.
  *
  * CPLError() will have been called if something fails.
  **********************************************************************/
-int TABDATFile::WriteLogicalField(const char *pszValue, TABINDFile *poINDFile,
+int TABDATFile::WriteLogicalField(bool bValue, TABINDFile *poINDFile,
                                   int nIndexNo)
 {
     if (m_poRecordBlock == nullptr)
@@ -2233,18 +2239,17 @@ int TABDATFile::WriteLogicalField(const char *pszValue, TABINDFile *poINDFile,
         return -1;
     }
 
-    // TODO(schwehr): bValue should be a bool.
-    const GByte bValue = STARTS_WITH_CI(pszValue, "T") ? 1 : 0;
+    const GByte byValue = bValue ? 1 : 0;
 
     // Update Index
     if (poINDFile && nIndexNo > 0)
     {
-        GByte *pKey = poINDFile->BuildKey(nIndexNo, static_cast<int>(bValue));
+        GByte *pKey = poINDFile->BuildKey(nIndexNo, static_cast<int>(byValue));
         if (poINDFile->AddEntry(nIndexNo, pKey, m_nCurRecordId) != 0)
             return -1;
     }
 
-    return m_poRecordBlock->WriteByte(bValue);
+    return m_poRecordBlock->WriteByte(byValue);
 }
 
 /**********************************************************************
