@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  gdaldem testing
@@ -21,20 +20,50 @@ import pytest
 
 from osgeo import gdal, osr
 
+###############################################################################
+# Test incorrect use of scale parameters
+
+
+@pytest.mark.parametrize(
+    "options",
+    [
+        {"scale": 111120, "xscale": 1},
+        {"scale": 111120, "yscale": 1},
+        {"xscale": 1},
+        {"yscale": 1},
+    ],
+)
+def test_gdaldem_lib_hillshade_incorrect_use_of_scale_params(options):
+
+    src_ds = gdal.Open("../gdrivers/data/n43.tif")
+    kwargs = {"format": "MEM", "zFactor": 30}
+    kwargs.update(options)
+    with pytest.raises(Exception):
+        gdal.DEMProcessing("", src_ds, "hillshade", **kwargs)
+
+
 ###############################################################################
 # Test gdaldem hillshade
 
 
-def test_gdaldem_lib_hillshade():
+@pytest.mark.parametrize(
+    "options,checksum",
+    [
+        ({"scale": 111120}, 45587),
+        ({"xscale": 111120, "yscale": 111120}, 45587),
+        ({"xscale": 111120, "yscale": 111120 + 1}, 45587),
+        ({}, 46008),
+    ],
+)
+def test_gdaldem_lib_hillshade(options, checksum):
 
     src_ds = gdal.Open("../gdrivers/data/n43.tif")
-    ds = gdal.DEMProcessing(
-        "", src_ds, "hillshade", format="MEM", scale=111120, zFactor=30
-    )
+    kwargs = {"format": "MEM", "zFactor": 30}
+    kwargs.update(options)
+    ds = gdal.DEMProcessing("", src_ds, "hillshade", **kwargs)
     assert ds is not None
 
-    cs = ds.GetRasterBand(1).Checksum()
-    assert cs == 45587, "Bad checksum"
+    assert ds.GetRasterBand(1).Checksum() == checksum, "Bad checksum"
 
     src_gt = src_ds.GetGeoTransform()
     dst_gt = ds.GetGeoTransform()
@@ -50,6 +79,87 @@ def test_gdaldem_lib_hillshade():
     ds = None
 
 
+###############################################################################
+# Test gdaldem hillshade with projected CRS
+
+
+@pytest.mark.parametrize("proj_unit", ["m", "ft", "us-ft"])
+def test_gdaldem_lib_hillshade_proj_crs(proj_unit):
+
+    src_ds = gdal.Warp(
+        "",
+        "../gdrivers/data/n43.tif",
+        format="MEM",
+        dstSRS=f"+proj=tmerc +datum=WGS84 +lon_0=-79.5 +units={proj_unit}",
+    )
+    ds = gdal.DEMProcessing("", src_ds, "hillshade", format="MEM", zFactor=30)
+    assert ds.GetRasterBand(1).Checksum() == 36233
+
+
+###############################################################################
+# Test gdaldem hillshade with projected CRS and different vertical unit
+
+
+@pytest.mark.parametrize(
+    "name,factor",
+    [("m", 1), ("unknown", 1), ("ft", 0.3048), ("us-ft", 0.3048006096012192)],
+)
+def test_gdaldem_lib_hillshade_projected_crs_vertical_units(tmp_path, name, factor):
+
+    gdaltest.importorskip_gdal_array()
+    pytest.importorskip("numpy")
+
+    src_ds = gdal.Warp(
+        "",
+        "../gdrivers/data/n43.tif",
+        outputType=gdal.GDT_Float32,
+        format="MEM",
+        dstSRS="+proj=tmerc +datum=WGS84 +lon_0=-79.5 +units=m",
+        dstNodata=0,
+    )
+    src_ds.GetRasterBand(1).SetUnitType(name)
+    src_ds.GetRasterBand(1).SetNoDataValue(
+        src_ds.GetRasterBand(1).GetNoDataValue() / factor
+    )
+    src_ds.GetRasterBand(1).WriteArray(src_ds.GetRasterBand(1).ReadAsArray() / factor)
+
+    if name == "unknown":
+        with gdal.quiet_errors():
+            ds = gdal.DEMProcessing("", src_ds, "hillshade", format="MEM", zFactor=30)
+    else:
+        ds = gdal.DEMProcessing("", src_ds, "hillshade", format="MEM", zFactor=30)
+    assert ds.GetRasterBand(1).Checksum() == 36233
+
+
+###############################################################################
+# Test gdaldem hillshade with geographic CRS and different vertical unit
+
+
+@pytest.mark.parametrize(
+    "name,factor",
+    [("m", 1), ("unknown", 1), ("ft", 0.3048), ("us-ft", 0.3048006096012192)],
+)
+def test_gdaldem_lib_hillshade_geographic_crs_vertical_units(name, factor):
+
+    gdaltest.importorskip_gdal_array()
+    pytest.importorskip("numpy")
+
+    src_ds = gdal.Translate(
+        "", "../gdrivers/data/n43.tif", format="MEM", outputType=gdal.GDT_Float32
+    )
+    src_ds.GetRasterBand(1).SetUnitType(name)
+    src_ds.GetRasterBand(1).SetNoDataValue(
+        src_ds.GetRasterBand(1).GetNoDataValue() / factor
+    )
+    src_ds.GetRasterBand(1).WriteArray(src_ds.GetRasterBand(1).ReadAsArray() / factor)
+    if name == "unknown":
+        with gdal.quiet_errors():
+            ds = gdal.DEMProcessing("", src_ds, "hillshade", format="MEM", zFactor=30)
+    else:
+        ds = gdal.DEMProcessing("", src_ds, "hillshade", format="MEM", zFactor=30)
+    assert ds.GetRasterBand(1).Checksum() == 46008
+
+
 ###############################################################################
 # Test gdaldem hillshade with source being floating point
 
@@ -128,16 +238,24 @@ def test_gdaldem_lib_hillshade_float_png(tmp_vsimem):
 # Test gdaldem hillshade -combined
 
 
-def test_gdaldem_lib_hillshade_combined():
+@pytest.mark.parametrize(
+    "options,checksum",
+    [
+        ({"scale": 111120}, 43876),
+        ({"xscale": 111120, "yscale": 111120}, 43876),
+        ({"xscale": 111120, "yscale": 111120 + 0.1}, 43876),
+        ({}, 42176),
+    ],
+)
+def test_gdaldem_lib_hillshade_combined(options, checksum):
 
     src_ds = gdal.Open("../gdrivers/data/n43.tif")
-    ds = gdal.DEMProcessing(
-        "", src_ds, "hillshade", format="MEM", combined=True, scale=111120, zFactor=30
-    )
+    kwargs = {"format": "MEM", "combined": True, "zFactor": 30}
+    kwargs.update(options)
+    ds = gdal.DEMProcessing("", src_ds, "hillshade", **kwargs)
     assert ds is not None
 
-    cs = ds.GetRasterBand(1).Checksum()
-    assert cs == 43876, "Bad checksum"
+    assert ds.GetRasterBand(1).Checksum() == checksum, "Bad checksum"
 
     src_gt = src_ds.GetGeoTransform()
     dst_gt = ds.GetGeoTransform()
@@ -157,22 +275,24 @@ def test_gdaldem_lib_hillshade_combined():
 # Test gdaldem hillshade -alg ZevenbergenThorne
 
 
-def test_gdaldem_lib_hillshade_ZevenbergenThorne():
+@pytest.mark.parametrize(
+    "options,checksum",
+    [
+        ({"scale": 111120}, 46544),
+        ({"xscale": 111120, "yscale": 111120}, 46544),
+        ({"xscale": 111120, "yscale": 111120 + 0.1}, 46544),
+        ({}, 45373),
+    ],
+)
+def test_gdaldem_lib_hillshade_ZevenbergenThorne(options, checksum):
 
     src_ds = gdal.Open("../gdrivers/data/n43.tif")
-    ds = gdal.DEMProcessing(
-        "",
-        src_ds,
-        "hillshade",
-        format="MEM",
-        alg="ZevenbergenThorne",
-        scale=111120,
-        zFactor=30,
-    )
+    kwargs = {"format": "MEM", "alg": "ZevenbergenThorne", "zFactor": 30}
+    kwargs.update(options)
+    ds = gdal.DEMProcessing("", src_ds, "hillshade", **kwargs)
     assert ds is not None
 
-    cs = ds.GetRasterBand(1).Checksum()
-    assert cs == 46544, "Bad checksum"
+    assert ds.GetRasterBand(1).Checksum() == checksum, "Bad checksum"
 
     src_gt = src_ds.GetGeoTransform()
     dst_gt = ds.GetGeoTransform()
@@ -192,23 +312,29 @@ def test_gdaldem_lib_hillshade_ZevenbergenThorne():
 # Test gdaldem hillshade -alg ZevenbergenThorne -combined
 
 
-def test_gdaldem_lib_hillshade_ZevenbergenThorne_combined():
+@pytest.mark.parametrize(
+    "options,checksum",
+    [
+        ({"scale": 111120}, 43112),
+        ({"xscale": 111120, "yscale": 111120}, 43112),
+        ({"xscale": 111120, "yscale": 111120 + 0.1}, 43112),
+        ({}, 42340),
+    ],
+)
+def test_gdaldem_lib_hillshade_ZevenbergenThorne_combined(options, checksum):
 
     src_ds = gdal.Open("../gdrivers/data/n43.tif")
-    ds = gdal.DEMProcessing(
-        "",
-        src_ds,
-        "hillshade",
-        format="MEM",
-        alg="ZevenbergenThorne",
-        combined=True,
-        scale=111120,
-        zFactor=30,
-    )
+    kwargs = {
+        "format": "MEM",
+        "alg": "ZevenbergenThorne",
+        "combined": True,
+        "zFactor": 30,
+    }
+    kwargs.update(options)
+    ds = gdal.DEMProcessing("", src_ds, "hillshade", **kwargs)
     assert ds is not None
 
-    cs = ds.GetRasterBand(1).Checksum()
-    assert cs == 43112, "Bad checksum"
+    assert ds.GetRasterBand(1).Checksum() == checksum, "Bad checksum"
 
     src_gt = src_ds.GetGeoTransform()
     dst_gt = ds.GetGeoTransform()
@@ -421,11 +547,11 @@ def test_gdaldem_lib_color_relief():
     )
     assert ds is not None
 
-    assert ds.GetRasterBand(1).Checksum() == 55009, "Bad checksum"
+    assert ds.GetRasterBand(1).Checksum() == 55066, "Bad checksum"
 
-    assert ds.GetRasterBand(2).Checksum() == 37543, "Bad checksum"
+    assert ds.GetRasterBand(2).Checksum() == 37594, "Bad checksum"
 
-    assert ds.GetRasterBand(3).Checksum() == 47711, "Bad checksum"
+    assert ds.GetRasterBand(3).Checksum() == 47768, "Bad checksum"
 
     src_gt = src_ds.GetGeoTransform()
     dst_gt = ds.GetGeoTransform()
@@ -463,7 +589,7 @@ def test_gdaldem_lib_color_relief():
         colorFilename="data/color_file.txt",
         colorSelection="linear_interpolation",
     )
-    assert ds.GetRasterBand(1).Checksum() == 55009
+    assert ds.GetRasterBand(1).Checksum() == 55066
 
     with pytest.raises(ValueError):
         gdal.DEMProcessing(
@@ -662,23 +788,24 @@ def test_gdaldem_lib_roughness():
 # Test gdaldem slope -alg ZevenbergenThorne
 
 
-def test_gdaldem_lib_slope_ZevenbergenThorne():
+@pytest.mark.parametrize(
+    "options,checksum",
+    [
+        ({"scale": 111120}, 64393),
+        ({"xscale": 111120, "yscale": 111120}, 64393),
+        ({"xscale": 111120, "yscale": 111120 + 0.1}, 64393),
+        ({}, 65468),
+    ],
+)
+def test_gdaldem_lib_slope_ZevenbergenThorne(options, checksum):
 
     src_ds = gdal.Open("../gdrivers/data/n43.tif")
-    ds = gdal.DEMProcessing(
-        "",
-        src_ds,
-        "slope",
-        format="MEM",
-        alg="ZevenbergenThorne",
-        slopeFormat="degree",
-        scale=111120,
-        zFactor=30,
-    )
+    kwargs = {"format": "MEM", "alg": "ZevenbergenThorne"}
+    kwargs.update(options)
+    ds = gdal.DEMProcessing("", src_ds, "slope", **kwargs)
     assert ds is not None
 
-    cs = ds.GetRasterBand(1).Checksum()
-    assert cs == 64393, "Bad checksum"
+    assert ds.GetRasterBand(1).Checksum() == checksum, "Bad checksum"
 
 
 ###############################################################################
@@ -783,3 +910,38 @@ def test_gdaldem_lib_dict_arguments():
     ind = opt.index("-co")
 
     assert opt[ind : ind + 4] == ["-co", "COMPRESS=DEFLATE", "-co", "LEVEL=4"]
+
+
+###############################################################################
+
+
+def flipped_ds():
+
+    src_ds = gdal.Open("../gdrivers/data/n43.tif")
+    gt = src_ds.GetGeoTransform()
+    ulx = "%.17g" % gt[0]
+    uly = "%.17g" % gt[3]
+    lrx = "%.17g" % (gt[0] + src_ds.RasterXSize * gt[1])
+    lry = "%.17g" % (gt[3] + src_ds.RasterYSize * gt[5])
+    return gdal.Warp("", src_ds, options=f"-of VRT -te {ulx} {uly} {lrx} {lry}")
+
+
+@pytest.mark.parametrize("alg", ["aspect", "TPI", "TRI"])
+def test_gdaldem_lib_flipped_aspect_tpi_tri(alg):
+
+    src_ds = gdal.Open("../gdrivers/data/n43.tif")
+    ref_ds = gdal.DEMProcessing("", src_ds, alg, format="MEM", computeEdges=True)
+    out_ds = gdal.DEMProcessing("", flipped_ds(), alg, format="MEM", computeEdges=True)
+    assert ref_ds.GetGeoTransform() == pytest.approx(out_ds.GetGeoTransform())
+    assert ref_ds.ReadRaster() == out_ds.ReadRaster()
+
+
+@pytest.mark.parametrize("alg", ["hillshade", "slope", "roughness"])
+def test_gdaldem_lib_flipped_hillshade_slope_roughness(alg):
+
+    src_ds = gdal.Open("../gdrivers/data/n43.tif")
+    ref_ds = gdal.DEMProcessing("", src_ds, alg, format="MEM", computeEdges=True)
+    out_ds = gdal.DEMProcessing("", flipped_ds(), alg, format="MEM", computeEdges=True)
+    out_ds = gdal.Warp("", out_ds, format="MEM")
+    assert ref_ds.GetGeoTransform() == pytest.approx(out_ds.GetGeoTransform())
+    assert ref_ds.ReadRaster() == out_ds.ReadRaster()
